%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% detect_creaky_voice_Dataset2XLS.m
%  This script runs the creaky voice detection algorithm developed by
%  Kane & Drugman over a set of audio files located at the INPUT_PATH
%  folder. Each audio file will be considered participant trials, and 
%  subfolders within will be considered different participant groups, 
%  with a tag name equals to the name of such folder. The script assumes
%  the MIC signal is at the channel n.1 if the audio file is mic/acc paired
%  data.
%
%  The Output consist of a spreadsheet .xlsx file, containing a list of all 
%  audio files with their respective % of creak time
%  relative to the total voicing time. The total voicing time in this
%  implementation is calculated by a simple 35 msec max-pool + thresholding 
%  procedure.
%
%  The "detect_creaky_voice()" function is part of the Covarep project, 
%  which can be found here: http://covarep.github.io/covarep 
%  For more information, check the function's help/comments by typing
%      >> help detect_creaky_voice
%
% Author: Manuel Diaz Cadiz, SteppLab, BU
% Date:   03/31/2022
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear; clc;
DEBUG = true;

% Define INPUT_PATH  
INPUT_PATH = 'R:\SteppLab3\Projects\Voice\Creak in AdLD\Full Rainbow';  % Change the INPUT FOLDER Here 
THRESHOLD  = 0.05; % Threshold (default 0.05) used for voice segmentation 

% GET group names from existing subfolders within INPUT_PATH
% (ignore folders that contains "!" / "." / "..")
GroupList = dir(INPUT_PATH);
GroupList = GroupList(arrayfun(@(x) x.isdir && ...
                                    ~strcmp(x.name,'.')  && ...
                                    ~strcmp(x.name,'..') && ...
                                    ~contains(x.name,'!') ,GroupList));
% For each Group
ctable = {'AudioFile' 'Group' 'Total Voiced time (sec)' 'Total Creak time (sec)' '% Creak' ...
          '' 'Data From:' INPUT_PATH};
tidx = 2;
for g=1:numel(GroupList)
    % Get SubjectList for this group
    SubjectList = dir(fullfile(GroupList(g).folder,GroupList(g).name));
    SubjectList = SubjectList(arrayfun(@(x) x.isdir && ...
                                    ~strcmp(x.name,'.')  && ...
                                    ~strcmp(x.name,'..') && ...
                                    ~contains(x.name,'!') ,SubjectList));
    for s=1:numel(SubjectList)
        % Get Subject's audio list
        AudioList = dir([fullfile(SubjectList(s).folder,SubjectList(s).name) '\*.wav']);
        % For Each Audio file
        for i=1:numel(AudioList)
            % Get audio signal
            this_audio_path = fullfile(AudioList(i).folder,AudioList(i).name);
            [y,fs0] = audioread(this_audio_path);
            if size(y,2)>1
                y = y(:,1);
            end
            % resample audio to 16Khz (required)
            fs = 16e3;
            y  = resample(y,fs,fs0);

            % Calculate Voicing mask
            Nw = 2*round(35e-3*fs/2)+1;
            y_maxpool = medfilt1(imclose(abs(y),ones(Nw,1)),round(Nw/2));
            y_maxpool = y_maxpool/prctile(y_maxpool,98);
            y_voiced  = y_maxpool>THRESHOLD;
            y_mask_voiced = imopen(y_voiced,ones(2*Nw,1));

            % Calculate Creak mask & interpolate to match audio signal size
            [~,crbb] = detect_creaky_voice(y,fs);
            t = (0:size(y,1)-1)'/fs;
            y_mask_creak = interp1(t(crbb(:,1)),crbb(:,2),t,'linear')>0.5;
            y_mask_creak = y_mask_creak & y_mask_voiced;

            % Calculate % creak for this audio file
            time_total  = sum(y_mask_voiced)/fs;
            time_creak  = sum(y_mask_creak)/fs;
            perct_creak = 100*time_creak/time_total;

            % Add Row to the table
            ctable(tidx,1:5) = {AudioList(i).name ...
                                GroupList(g).name ...
                                time_total ...
                                time_creak ...
                                perct_creak};
            tidx = tidx + 1;

            if DEBUG
                figure(1); clf;
                plot(y);
                hold on;
                plot(y_maxpool);
                plot(y_mask_voiced);
                plot(y_mask_creak);
                drawnow;
            end      
        end
    end
end
outTable = cell2table(ctable);
writetable(outTable,['CreakDetection_output_' datestr(clock, 'yyyymmdd_HHMMSS') '.xlsx'],'WriteVariableNames',false);


